@@ -9,6 +9,10 @@ DATA_DIR = os.path.join(
     "project"
 )
 
+FORCED_UNIX = os.environ.get("BST_FORCE_BACKEND") == "unix"
+
+
+@pytest.mark.skipif(FORCED_UNIX, reason="Refusing to run on forced unix backend")
 @pytest.mark.datafiles(DATA_DIR)
 def test_dpkg_build(cli, datafiles):
     project = str(datafiles)
@@ -23,6 +27,7 @@ def test_dpkg_build(cli, datafiles):
     assert_contains(checkout_dir, ['/usr/share/foo', '/usr/share/doc/test/changelog.gz'])
 
 
+@pytest.mark.skipif(FORCED_UNIX, reason="Refusing to run on forced unix backend")
 @pytest.mark.datafiles(DATA_DIR)
 def test_dpkg_deploy(cli, datafiles):
     project = str(datafiles)
