@@ -14,14 +14,14 @@ def plugin_import(datafiles):
     sources_dst = os.path.join(plugins_dir, "sources")
     os.makedirs(elements_dst, exist_ok=True)
     for f in os.listdir(elements_src):
-        if f is not "__init__.py" and (f.endswith(".py") or f.endswith(".yaml")):
+        if f != "__init__.py" and (f.endswith(".py") or f.endswith(".yaml")):
             fsrc = os.path.join(elements_src, f)
             fdst = os.path.join(elements_dst ,f)
             shutil.copyfile(fsrc, fdst)
 
     os.makedirs(sources_dst, exist_ok=True)
     for f in os.listdir(sources_src):
-        if f is not "__init__.py" and (f.endswith(".py") or f.endswith(".yaml")):
+        if f != "__init__.py" and (f.endswith(".py") or f.endswith(".yaml")):
             fsrc = os.path.join(sources_src, f)
             fdst = os.path.join(sources_dst ,f)
             shutil.copyfile(fsrc, fdst)
